@@ -1,7 +1,7 @@
 /*  XMMS - Cross-platform multimedia player
  *  Copyright (C) 1998-1999  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
  *
- *  FM Radio plug-in v1.3
+ *  FM Radio plug-in v1.5
  *  Copyright (C) 1999 Julien Viard <Julien.Viard@enst.fr>
  *  
  *  Radio acces routines from Keith Wesolowski (wesolows@cs.unr.edu)
@@ -25,9 +25,9 @@
 #include "radio.h"
 
 #include <errno.h>
-#ifdef __OpenBSD__
+#if defined __OpenBSD__ || defined __NetBSD__
 #include <err.h>
-#endif /* __OpenBSD__ */
+#endif /* __OpenBSD__ || __NetBSD__ */
 
 static void radio_init(void);
 static int is_our_file(char *filename);
@@ -44,7 +44,7 @@ InputPlugin radio_ip =
 {
   NULL,
   NULL,
-  "FM Radio player 1.4",
+  "FM Radio player 1.5",
   radio_init,
   radio_about,
   radio_configure,
@@ -72,10 +72,12 @@ RadioConfig radio_cfg;
 
 gint radio_fd = -1;
 gint freq,volume;
-#ifdef __OpenBSD__
+#ifdef BSD_RADIO
 struct radio_info ri;
+#endif /* BSD_RADIO */
+#if defined __OpenBSD__ || defined __NetBSD__
 static mixer_ctrl_t volctl;
-#endif /* __OpenBSD__ */
+#endif /* __OpenBSD__ || __NetBSD__ */
 gboolean is_paused,tuned;
 
 
@@ -98,7 +100,7 @@ static void radio_init(void)
   radio_cfg.use_oss_mixer = TRUE;
 #endif
   
-  if (cfgfile = xmms_cfg_open_file(filename))
+  if ((cfgfile = xmms_cfg_open_file(filename)) != NULL)
     {
       xmms_cfg_read_string(cfgfile, "Radio", "device", &radio_cfg.device);
       xmms_cfg_read_string(cfgfile, "Radio", "directory", &radio_cfg.directory);
@@ -110,16 +112,16 @@ static void radio_init(void)
   if (!radio_cfg.device)
 #ifdef linux
     radio_cfg.device = g_strdup("/dev/radio0");
-#elif defined __OpenBSD__
+#elif defined BSD_RADIO
     radio_cfg.device = g_strdup("/dev/radio");
-#endif /* linux || __OpenBSD__ */
+#endif /* linux || BSD_RADIO */
   if (!radio_cfg.directory)
     radio_cfg.directory = g_strdup(g_get_home_dir());
   
   volume=FMR_VOL_DEFAULT;
-#ifdef __OpenBSD__
-  SetGetRadioInfo(GET_INFO);
-#endif /* __OpenBSD__ */
+#ifdef BSD_RADIO
+  SetGetRadioInfo(RIOCGINFO);
+#endif /* BSD_RADIO */
 }
 
 static int is_our_file(char *filename)
@@ -161,19 +163,19 @@ double fmradio_get_freq_fact (void)
     return .016;
   return 16;
 }
-#elif defined __OpenBSD__
+#elif defined BSD_RADIO
 int
 SetGetRadioInfo(int setinfo) {
 	int rd;
 
-	rd = open(radio_cfg.device, O_RDONLY);
+	rd = open(radio_cfg.device, O_RDWR);
 	if (rd < 0) {
 		warn("%s open error", radio_cfg.device);
 		return -1;
 	}
 
-	if (ioctl(rd, setinfo == SET_INFO ? RIOCSINFO : RIOCGINFO, &ri) < 0) {
-		warn("%s", setinfo == SET_INFO ? "RIOCSINFO" : "RIOCGINFO");
+	if (ioctl(rd, setinfo == RIOCSINFO ? RIOCSINFO : RIOCGINFO, &ri) < 0) {
+		warn("%s", setinfo == RIOCSINFO ? "RIOCSINFO" : "RIOCGINFO");
 		return -1;
 	}
 
@@ -182,7 +184,7 @@ SetGetRadioInfo(int setinfo) {
 
 	return 0;
 }
-#endif /* linux || __OpenBSD__ */
+#endif /* linux || BSD_RADIO */
 
 
 static void play_file(char *filename)
@@ -200,7 +202,8 @@ static void play_file(char *filename)
   if(file!=NULL){
     radioname=(gchar*)malloc(STR_LEN_MAX*sizeof(gchar));
     fgets(radioname,STR_LEN_MAX,file);
-    if (tmp = strrchr(radioname, '\n'))
+    radioname[STR_LEN_MAX-1] = '\0';
+    if ((tmp = strrchr(radioname, '\n')) != NULL)
       *tmp = '\0';
     if(!fscanf(file,"Freq:%d", &freq)) freq=0;
     fclose(file);
@@ -220,7 +223,7 @@ static void play_file(char *filename)
       freq=0;
       if(radioname==NULL){
 	radioname = g_strdup(filename);
-	if (tmp = strrchr(radioname, '.'))
+	if ((tmp = strrchr(radioname, '.')) != NULL)
 	  *tmp = '\0';
       }
     }
@@ -268,15 +271,15 @@ static void play_file(char *filename)
       return ;
     }
     tuned = ((v.signal != 0) ? TRUE : FALSE);
-#elif defined __OpenBSD__
+#elif defined BSD_RADIO
     ri.freq = (unsigned long)freq;
     ri.mute = 0;
     ri.volume = volume * (255/(FMR_VOL_MAX - FMR_VOL_MIN));
-    SetGetRadioInfo(SET_INFO);
+    SetGetRadioInfo(RIOCSINFO);
     usleep(50000);
-    SetGetRadioInfo(GET_INFO);
+    SetGetRadioInfo(RIOCGINFO);
     tuned = ri.info ? TRUE : FALSE;
-#endif /* linux || __OpenBSD__ */
+#endif /* linux || BSD_RADIO */
     
     if (radioname==NULL)
       tmp = g_strdup_printf("FM Radio %6.2fMHz", (float)freq/1000.0);
@@ -316,15 +319,15 @@ static void play_file(char *filename)
       return ;
     }
     tuned = ((v.signal != 0) ? TRUE : FALSE);
-#elif defined __OpenBSD__
+#elif defined BSD_RADIO
     ri.freq = (unsigned long)freq;
     ri.mute = 0;
     ri.volume = volume * (255/(FMR_VOL_MAX - FMR_VOL_MIN));
-    SetGetRadioInfo(SET_INFO);
+    SetGetRadioInfo(RIOCSINFO);
     usleep(50000);
-    SetGetRadioInfo(GET_INFO);
+    SetGetRadioInfo(RIOCGINFO);
     tuned = ri.info ? TRUE : FALSE;
-#endif /* linux || __OpenBSD__ */
+#endif /* linux || BSD_RADIO */
 
     if (radioname==NULL)
       tmp = g_strdup_printf("FM Radio (undefined)");
@@ -350,10 +353,10 @@ static void stop(void)
   ioctl (radio_fd, VIDIOCSAUDIO, &va);
   close(radio_fd);
   radio_fd = -1;
-#elif defined __OpenBSD__
+#elif defined BSD_RADIO
   ri.mute = 1;
-  SetGetRadioInfo(SET_INFO);
-#endif /* linux || __OpenBSD__ */
+  SetGetRadioInfo(RIOCSINFO);
+#endif /* linux || BSD_RADIO */
 }
 
 static void radio_pause(short p)
@@ -380,7 +383,7 @@ static void radio_pause(short p)
   is_paused = (p ? TRUE : FALSE);
   close(radio_fd);
   radio_fd = -2;
-#elif defined __OpenBSD__
+#elif defined BSD_RADIO
   if (p) {
 	  ri.mute = 1;
 	  is_paused = TRUE;
@@ -388,13 +391,13 @@ static void radio_pause(short p)
 	  ri.mute = 0;
 	  is_paused = FALSE;
   }
-  SetGetRadioInfo(SET_INFO);
-#endif /* linux || __OpenBSD__ */
+  SetGetRadioInfo(RIOCSINFO);
+#endif /* linux || BSD_RADIO */
 }
 
 static void get_song_info(char *filename, char **title, int *len)
 {
-  gint fr, nt;
+  gint fr;
   gchar *tmp,*radioname;
   FILE * file;
   
@@ -407,7 +410,8 @@ static void get_song_info(char *filename, char **title, int *len)
   if(file!=NULL){
     radioname=(gchar*)malloc(STR_LEN_MAX*sizeof(gchar));
     fgets(radioname,STR_LEN_MAX,file);
-    if (tmp = strrchr(radioname, '\n'))
+    radioname[STR_LEN_MAX-1] = '\0';
+    if ((tmp = strrchr(radioname, '\n')) != NULL)
       *tmp = '\0';
     if(!fscanf(file,"Freq:%d", &fr)) fr=0;
     fclose(file);
@@ -426,7 +430,7 @@ static void get_song_info(char *filename, char **title, int *len)
       fr=0;
       if(radioname==NULL){
 	radioname = g_strdup(filename);
-	if (tmp = strrchr(radioname, '.'))
+	if ((tmp = strrchr(radioname, '.')) != NULL)
 	  *tmp = '\0';
       }
     }
@@ -453,10 +457,12 @@ static void get_song_info(char *filename, char **title, int *len)
 
 static void get_volume(int *l, int *r)
 {
-#ifdef __OpenBSD__
+#ifdef linux
+	int v;
+#elif defined __OpenBSD__ || defined __NetBSD__
 	mixer_devinfo_t devinfo;
-#endif /* __OpenBSD__ */
-  int fd, devs, cmd, v;
+#endif /* linux || __OpenBSD__ || __NetBSD__ */
+  int fd, devs, cmd;
 
 #if defined(HAVE_SYS_SOUNDCARD_H) || defined(HAVE_MACHINE_SOUNDCARD_H)
   if (radio_cfg.use_oss_mixer)
@@ -490,7 +496,7 @@ static void get_volume(int *l, int *r)
 	} else {
 	  printf( "Cannot open /dev/mixer %ld\n", errno );
 	}
-#elif defined __OpenBSD__
+#elif defined __OpenBSD__ || defined __NetBSD__
 	cmd = 1;
 	fd = open("/dev/mixer", O_RDWR);
 	if (fd < 0) {
@@ -537,7 +543,7 @@ static void get_volume(int *l, int *r)
 
 	if (close(fd) < 0)
 		warn("/dev/mixer close error");
-#endif /* linux || __OpenBSD__ */
+#endif /* linux || __OpenBSD__ || __NetBSD__ */
     }
   else
 #endif
@@ -555,10 +561,11 @@ static void set_volume(int l, int r)
 {
 #ifdef linux
   struct video_audio va;
-#elif defined __OpenBSD__
+  int v;
+#elif defined __OpenBSD__ || defined __NetBSD__
   mixer_devinfo_t devinfo;
-#endif /* linux || __OpenBSD__ */
-  int fd, devs, cmd, v;
+#endif /* linux || __OpenBSD__ || __NetBSD__ */
+  int fd, devs, cmd;
   
 #if defined(HAVE_SYS_SOUNDCARD_H) || defined(HAVE_MACHINE_SOUNDCARD_H)
   if (radio_cfg.use_oss_mixer)
@@ -590,7 +597,7 @@ static void set_volume(int l, int r)
 	} else {
 	  printf( "Cannot open /dev/mixer %ld\n", errno );
 	}
-#elif defined __OpenBSD__
+#elif defined __OpenBSD__ || defined __NetBSD__
 	cmd = 1;
 	fd = open("/dev/mixer", O_RDWR);
 	if (fd < 0) {
@@ -639,7 +646,7 @@ static void set_volume(int l, int r)
 
 	if (close(fd) < 0)
 		warn("/dev/mixer close error");
-#endif /* linux || __OpenBSD__ */
+#endif /* linux || __OpenBSD__ || __NetBSD__ */
     }
   else
 #endif
@@ -676,11 +683,11 @@ static void set_volume(int l, int r)
       }
       close(radio_fd);
       radio_fd = -2;
-#elif defined __OpenBSD__
+#elif defined BSD_RADIO
     ri.volume = volume * (255 / (FMR_VOL_MAX - FMR_VOL_MIN));
     ri.mute = is_paused ? 1 : 0;
-    SetGetRadioInfo(SET_INFO);
-#endif /* linux || __OpenBSD__ */
+    SetGetRadioInfo(RIOCSINFO);
+#endif /* linux || BSD_RADIO */
     }
 }
 
@@ -695,7 +702,7 @@ static void radio_about(void)
 		  "Thanks to\n"
 		  " Keith Wesolowski <wesolows@cs.unr.edu> \nfor the program GRadio 1.0.0\n"
 		  " Wim <wim.delvaux@chello.be> \nfor correcting my code for the volume configuration through the radio device\n"
-		  " Vladimir Popov <jumbo@narod.ru>\n\tfor the BSD port\n"
+		  " Vladimir Popov <jumbo@narod.ru>\nfor the BSD port\n"
 		  "\nTo use it, open a file with '.fmr' extension\n then edit the 'file info' to scan for a station and save the file.", "Ok",
 		FALSE, NULL, NULL);
 	gtk_signal_connect(GTK_OBJECT(box), "destroy",
